package com.sillycycle.bagleyd.threed;

/*
 * @(#)Threed.java
 *
 * Copyright 1994 - 2022  David A. Bagley, bagleyd AT verizon.net
 *
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of the author not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 *
 * This program is distributed in the hope that it will be "useful",
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

import java.awt.Color;
import java.awt.Cursor;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import javax.swing.JFrame;

/**
 * The <code>Threed</code> class extends the
 * <code>ThreedCanvas</code> for handling the responses to the user
 * for the <code>ThreedFrame</code> class.
 *
 * @author	David A. Bagley
 * @author	bagleyd AT verizon.net
 * @author	https://www.sillycycle.com/threed.html
 */

public class Threed extends ThreedCanvas implements MouseWheelListener,
		MouseListener, MouseMotionListener, KeyListener {
	private static final long serialVersionUID = 42L;

	public Threed(JFrame frame,
		int object, boolean surface,
		int x, int y, int z,
		int theta, int phi, int psi,
		boolean mono, boolean reverse, String names,
		Color c0, Color c1, Color c2,
		Color c3, Color c4, Color c5,
		Color borderc, Color fg, Color bg, int delay) {

		this.frame = frame;
		this.object = object;
		this.surface = surface;
		this.distance = new ThreedPoint(x, y, z);
		this.angle = new ThreedAngle(theta, phi, psi);
		this.deltaAngle = new ThreedAngle(theta, phi, psi);
		this.mono = mono;
		this.reverse = reverse;
		this.shadeNames = names;
		this.shadeColor[0] = c0;
		this.shadeColor[1] = c1;
		this.shadeColor[2] = c2;
		this.shadeColor[3] = c3;
		this.shadeColor[4] = c4;
		this.shadeColor[5] = c5;
		this.currentForeground = this.borderColor = borderc;
		this.foreground = fg; this.background = bg;
		this.delay = delay;
		firstPaint = true;
		this.addMouseListener(this);
		this.addMouseMotionListener(this);
		this.addMouseWheelListener(this);
		this.addKeyListener(this);
		setCursor(new Cursor(Cursor.HAND_CURSOR));
	}

	@Override
	public void mousePressed(MouseEvent event) {
		selectThreeD(event.getX(), event.getY());
		exposeThreeD();
	}

	@Override
	public void mouseClicked(MouseEvent event) {
		/* Method required but mousePressed does work */
	}

	@Override
	public void mouseReleased(MouseEvent event) {
		if (mouseDown) { // Do not care which mouse button
			releaseThreeD();
			mouseDown = false;
		}
		currentPosition.x = 2 * coreSize.x;
		currentPosition.y = 2 * coreSize.y;
	}

	@Override
	public void mouseEntered(MouseEvent event) {
		currentForeground = foreground;
		framePaint = true;
		mouseDown = false;
		repaint();
	}

	@Override
	public void mouseExited(MouseEvent event) {
		if (mouseDown) {
			releasePaint = true;
		}
		currentForeground = borderColor;
		framePaint = true;
		mouseDown = false;
		repaint();
	}

	@Override
	public void mouseDragged(MouseEvent event) {
		motionThreeD(event.getX(), event.getY());
	}

	@Override
	public void mouseMoved(MouseEvent event) {
		/* Method required but mousePressed does work */
	}

	@Override
	public void mouseWheelMoved(MouseWheelEvent event) {
		int notches = event.getWheelRotation();

		if (notches < 0) {
			moveRaisePhiThreeD();
		} else {
			moveLowerPhiThreeD();
		}
	}

	@Override
	public void keyPressed(KeyEvent event) {
		switch (event.getKeyCode()) {
		case KeyEvent.VK_ESCAPE:
			((ThreedFrame) frame).shuffleDown();
			return;
		case KeyEvent.VK_F11:
		case KeyEvent.VK_DIVIDE:
			moveRaisePsiThreeD();
			return;
		case KeyEvent.VK_UP:
		case KeyEvent.VK_KP_UP:
		case KeyEvent.VK_NUMPAD8:
			moveRaisePhiThreeD();
			return;
		case KeyEvent.VK_LEFT:
		case KeyEvent.VK_KP_LEFT:
		case KeyEvent.VK_NUMPAD4:
			moveRaiseThetaThreeD();
			return;
		case KeyEvent.VK_F12:
		case KeyEvent.VK_CLEAR:
			moveLowerPsiThreeD();
			return;
		case KeyEvent.VK_RIGHT:
		case KeyEvent.VK_KP_RIGHT:
		case KeyEvent.VK_NUMPAD6:
			moveLowerThetaThreeD();
			return;
		case KeyEvent.VK_DOWN:
		case KeyEvent.VK_KP_DOWN:
		case KeyEvent.VK_NUMPAD2:
			moveLowerPhiThreeD();
			return;
		default:
			processKey(event.getKeyChar());
		}
	}

	public boolean processKey(char key) {
		switch (key) {
		case 's':
		case 'S':
			surfaceThreeD();
			break;
		case 'b':
		case 'B':
			objectThreeD();
			break;
		case '?':
			((ThreedFrame) frame).descriptionHelp();
			return true;
		case '!':
			((ThreedFrame) frame).featuresHelp();
			return true;
		case '^':
			((ThreedFrame) frame).referencesHelp();
			return true;
		case 'a':
		case 'A':
			((ThreedFrame) frame).aboutHelp();
			return true;
		case '/':
			moveRaisePsiThreeD();
			break;
		case '8':
			moveRaisePhiThreeD();
			break;
		case '4':
			moveRaiseThetaThreeD();
			break;
		case '5':
			moveLowerPsiThreeD();
			break;
		case '6':
			moveLowerThetaThreeD();
			break;
		case '2':
			moveLowerPhiThreeD();
			break;
		case 'l':
			moveLeftThreeD();
			break;
		case 'r':
			moveRightThreeD();
			break;
		case 'u':
			moveUpThreeD();
			break;
		case 'd':
			moveDownThreeD();
			break;
		case 'o':
			moveOutThreeD();
			break;
		case 'i':
			moveInThreeD();
			break;
		case 'q':
		case 'Q':
		case 'x':
		case 'X':
		case '\03': /* Control-C */
			((ThreedFrame) frame).quit();
			break;
		default:
			return false;
		}
		return true;
	}

	@Override
	public void keyReleased(KeyEvent event) {
		/* Method required but keyPressed does work */
	}

	@Override
	public void keyTyped(KeyEvent event) {
		/* Method required but keyPressed does work */
	}
}
