package com.sillycycle.bagleyd.threed;

/*
 * @(#)ThreedMatrix.java
 *
 * Copyright 2005 - 2014  David A. Bagley, bagleyd AT verizon.net
 *
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of the author not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 *
 * This program is distributed in the hope that it will be "useful",
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

/**
 * The <code>ThreedMatrix</code> class handles the matrix
 * manipulation for the <code>ThreedFrame</code> class.
 *
 * @author	David A. Bagley
 * @author	bagleyd AT verizon.net
 * @author	https://www.sillycycle.com/threed.html
 */

public class ThreedMatrix {
	double[][] a = new double[4][4];

	void setInMatrix(int x, int y, double z) {
		a[x][y] = z;
	}

	double getInMatrix(int x, int y) {
		return a[x][y];
	}

	void identityMatrix4x4() {
		a[0][1] = a[0][2] = a[0][3] = 0.0;
		a[1][0] = a[1][2] = a[1][3] = 0.0;
		a[2][0] = a[2][1] = a[2][3] = 0.0;
		a[3][0] = a[3][1] = a[3][2] = 0.0;
		/* diagonal of 1's */
		a[0][0] = a[1][1] = a[2][2] = a[3][3] = 1.0;
	}

	void zeroMatrix4x4() {
		a[0][0] = a[0][1] = a[0][2] = a[0][3] = 0.0;
		a[1][0] = a[1][1] = a[1][2] = a[1][3] = 0.0;
		a[2][0] = a[2][1] = a[2][2] = a[2][3] = 0.0;
		a[3][0] = a[3][1] = a[3][2] = a[3][3] = 0.0;
	}

	void copyMatrix4x4(ThreedMatrix source) {
		int i, j;

		for (i = 0; i < 4; i++)
			for (j = 0; j < 4; j++)
				a[i][j] = source.a[i][j];
	}

	void printMatrix4x4() {
		int i, j;

		for (j = 0; j < 4; j++) {
			for (i = 0; i < 4; i++)
				System.out.print(a[i][j] + " ");
			System.out.println("");
		}
	}

	/*void printMatrix1x4() {
		int i;

		for (i = 0; i < 4; i++)
			System.out.print(a[i] + " ");
		System.out.println("");
	}*/

	void multiplyMatrix4x4_4x4(ThreedMatrix b, ThreedMatrix c) {
		/*
		int i, j, k;
		double sum;

		for (i = 0; i < 4; i++)
			for (j = 0; j < 4; j++) {
				sum = 0.0;
				for (k = 0; k < 4; k++)
					sum += b[i][k] * c[k][j];
				a[i][j] = sum;
			}
		*/
		/* sparse aware, not for general use */
		double b00 = b.a[0][0], b01 = b.a[0][1], b02 = b.a[0][2];
		double b10 = b.a[1][0], b11 = b.a[1][1], b12 = b.a[1][2];
		double b20 = b.a[2][0], b21 = b.a[2][1], b22 = b.a[2][2];
		double c00 = c.a[0][0], c01 = c.a[0][1], c02 = c.a[0][2];
		double c10 = c.a[1][0], c11 = c.a[1][1], c12 = c.a[1][2];
		double c20 = c.a[2][0], c21 = c.a[2][1], c22 = c.a[2][2];

		a[0][0] = b00 * c00 + b01 * c10 + b02 * c20;
		a[0][1] = b00 * c01 + b01 * c11 + b02 * c21;
		a[0][2] = b00 * c02 + b01 * c12 + b02 * c22;
		a[1][0] = b10 * c00 + b11 * c10 + b12 * c20;
		a[1][1] = b10 * c01 + b11 * c11 + b12 * c21;
		a[1][2] = b10 * c02 + b11 * c12 + b12 * c22;
		a[2][0] = b20 * c00 + b21 * c10 + b22 * c20;
		a[2][1] = b20 * c01 + b21 * c11 + b22 * c21;
		a[2][2] = b20 * c02 + b21 * c12 + b22 * c22;
		a[0][3] = a[1][3] = a[2][3] = 0.0;
		a[3][0] = a[3][1] = a[3][2] = 0.0;
		a[3][3] = 1.0;
	}
/*
	void multiplyMatrix1x4_4x4(Matrix1x4 a, Matrix4x4 b, Matrix1x4 result)
	{
		int j, k;
		double sum;

		for (j = 0; j < 4; j++) {
			sum = 0.0;
			for (k = 0; k < 4; k++)
				sum += a[k] * b[k][j];
			result[j] = sum;
		}
	}*/
}
