package com.sillycycle.bagleyd.threed;

/*
 * @(#)ThreedVector.java
 *
 * Copyright 2005 - 2014  David A. Bagley, bagleyd AT verizon.net
 *
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of the author not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 *
 * This program is distributed in the hope that it will be "useful",
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

/**
 * The <code>ThreedVector</code> class handles the vectors to be
 * used for the <code>ThreedFrame</code> class.
 *
 * @author	David A. Bagley
 * @author	bagleyd AT verizon.net
 * @author	https://www.sillycycle.com/threed.html
 */

public class ThreedVector {
	double      x, y, z;

	public ThreedVector(double x, double y, double z) {
		this.x = x;
		this.y = y;
		this.z = z;
	}

	/* Black Art of 3D Game Programming Andre LaMothe */
	public ThreedVector(ThreedPoint init, ThreedPoint term) {
		/* this creates a vector from 2 points in 3D space */
		this.x = term.x - init.x;
		this.y = term.y - init.y;
		this.z = term.z - init.z;
	}

	double magnitudeVector3D() {
		return Math.sqrt(this.x * this.x + this.y * this.y +
			this.z * this.z);
	}

	double dotProduct3D(ThreedVector v) {
		return (this.x * v.x + this.y * v.y + this.z * v.z);
	}

	ThreedVector crossProduct3D(ThreedVector v) {
		/* left hand rule */
		return new ThreedVector(
			this.y * v.z - this.z * v.y,
			this.z * v.x - this.x * v.z,
			this.x * v.y - this.y * v.x);
	}

	ThreedVector addProduct3D(ThreedVector v) {
		return new ThreedVector(
			this.x + v.x,
			this.y + v.y,
			this.z + v.z);
	}

	ThreedVector subtractProduct3D(ThreedVector v) {
		return new ThreedVector(
			this.x - v.x,
			this.y - v.y,
			this.z - v.z);
	}
}
